#!/usr/bin/env python3
# pyright: reportOptionalMemberAccess=false
import time
import traceback

from colorama import Fore, Style

from .tournament_client import TournamentConnection


class ColosseumTournamentAgent:
    def __init__(
        self,
        policy,
        latency=0,
        games=["__all__"],
        port=12345,
        direct=False,
        host="localhost",
        maximum_rounds=10000,
        debug=False,
    ):
        self.host = host
        self.policy = policy
        self.port = port
        self.botname = ""
        self.key = ""
        self.latency = latency
        self.direct = direct
        self.games = games
        self.tournament = None
        self.maximum_rounds = maximum_rounds
        self.debug = debug

    def connect(self, botname, key):
        self.botname = botname
        self.key = key
        try:
            # TODO: Use policy for test
            # Test that policy runs without errors in local environments
            self.tournament = TournamentConnection(
                self.host,
                self.port,
                self.botname,
                self.key,
                available_games=self.games,
                debug=self.debug,
            )
            print(
                Fore.GREEN
                + f"Bot: {self.botname}'s policy has passed environment verifications"
            )
            print(Style.RESET_ALL)
        except Exception:
            print(
                Fore.RED
                + f"Bot: {self.botname}'s policy has failed verification testing in environment: {self.games}"
            )
            print(Style.RESET_ALL)
            quit()

    def run(self):
        # Connect to tournament server each round, until the end signal is received.
        try:
            env = self.tournament.next_match()
            if self.debug:
                print(env)
        except Exception as e:
            print(Fore.RED + str(e))
            if self.debug:
                print(traceback.format_exc())
            print(Style.RESET_ALL)
            quit()
        current_round = 1
        while env is not None:
            term = False
            trunc = False
            timestep = 0
            rew = info = None
            obs = env.reset()
            while not (term or trunc):
                if timestep % 100 == 0 and self.debug:
                    print(f"{self.botname}: Timestep {timestep}\n")
                time.sleep(self.latency / 1000)  # Used to simulate network latency
                action_surprise = self.policy(
                    obs, rew, term, trunc, info
                )  # receive action and surprise from user
                obs, rew, term, trunc, info = env.step(
                    action_surprise
                )  # Send action to game server
                timestep += 1
            print(Fore.GREEN + f"Round {current_round} complete")
            print(Style.RESET_ALL)
            current_round += 1
            if current_round > self.maximum_rounds:
                env = None
            else:
                env = self.tournament.next_match()



import collections
import json


def braces_match(json_str):
    return json_str.count(b"{") == json_str.count(b"}")


class SocketWrapper:
    def __init__(self, socket):
        self.socket = socket
        self.received_data = collections.deque()
        self.unparsed_data = b""

    def recv(self, buffer_size):
        while not self.received_data:
            new_data = self.socket.recv(buffer_size)
            self.unparsed_data = self.unparsed_data + new_data
            start_idx = 0
            newbreak = self.unparsed_data.find(b"}{", start_idx)
            while newbreak != -1:
                end_idx = newbreak + 1
                data_chunk = self.unparsed_data[start_idx:end_idx]
                self.received_data.append(data_chunk)

                start_idx = end_idx
                newbreak = self.unparsed_data.find(b"}{", start_idx)

            final_chunk = self.unparsed_data[start_idx:]
            if braces_match(final_chunk):
                self.unparsed_data = b""
                self.received_data.append(final_chunk)
            else:
                self.unparsed_data = final_chunk

        retbytes = self.received_data.popleft()
        retdata = json.loads(retbytes.decode("utf-8"))
        return retdata

    def __getattr__(self, val):
        return getattr(self.socket, val)



import numpy as np

from umshini.utils.compress import compress, decompress

input_array = np.random.uniform(low=-9, high=9, size=(4, 4))
print(input_array)
compressed_array = compress(input_array)
print(compressed_array)
decompressed_array = decompress(compressed_array)
print(decompressed_array)
assert np.array_equal(input_array, decompressed_array)
print("Test Passed")
